# Equivalents to 'use warnings; use strict;'
[TestingAndDebugging::RequireUseStrict]
equivalent_modules = strictures Moo Moo::Role
[TestingAndDebugging::RequireUseWarnings]
equivalent_modules = strictures Moo Moo::Role
